#!/usr/bin/env node
'use strict';

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _meow = require('meow');

var _meow2 = _interopRequireDefault(_meow);

var _mkdirp = require('mkdirp');

var _mkdirp2 = _interopRequireDefault(_mkdirp);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _standalone = require('./standalone');

var _standalone2 = _interopRequireDefault(_standalone);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var cli = (0, _meow2.default)('\n    Usage\n        $ webfont <path|glob> ... [options]\n    Options\n        General:\n            -f, --font-name            The font family name you want, default: "webfont".\n            -h, --help                 Output usage information.\n            -v, --version              Output the version number.\n            --formats                  Only this formats generate.\n            --dest                     Destination for generated fonts.\n            --src-css-template         Path to custom template.\n            --css-template-class-name  Class name in css template.\n            --css-template-font-path   Font path in css template.\n            --css-template-font-name   Font name in css template.\n            --dest-css-template        Destination for generated css template.\n            --verbose                  Tell me everything!.\n\n        For "svgicons2svgfont":\n            -i, --font-id              The font id you want, default as "--font-name".\n            -t, --style                The font style you want.\n            -g, --weight               The font weight you want.\n            -w, --fixed-width          Creates a monospace font of the width of the largest input icon.\n            -c, --center-horizontally  Calculate the bounds of a glyph and center it horizontally.\n            -n, --normalize            Normalize icons by scaling them to the height of the highest icon.\n            -e, --height               The outputted font height [MAX(icons.height)].\n            -r, --round                Setup the SVG path rounding [10e12].\n            -d, --descent              The font descent [0].\n            -a, --ascent               The font ascent [height - descent].\n            -s, --start-unicode        The start unicode codepoint for unprefixed files [0xEA01].\n            -p, --prepend-unicode      Prefix files with their automatically allocated unicode codepoint.\n            -m, --metadata             Content of the metadata tag.\n', {
    string: ['font-name', 'dest', 'src-css-template', 'css-template-class-name', 'css-template-font-path', 'css-template-font-name', 'dest-css-template', 'font-id', 'style', 'weight', 'height', 'round', 'descent', 'ascent', 'start-unicode'],
    boolean: ['css', 'help', 'version', 'verbose', 'fixed-width', 'center-horizontally', 'normalize', 'prepend-unicode'],
    /* eslint-disable id-length */
    alias: {
        h: 'help',
        v: 'version',
        f: 'font-name',
        i: 'font-id',
        t: 'style',
        g: 'weight',
        w: 'fixed-width',
        c: 'center-horizontally',
        n: 'normalize',
        e: 'height',
        r: 'round',
        d: 'descent',
        a: 'ascent',
        s: 'start-unicode',
        p: 'prepend-unicode',
        m: 'metadata'
    }
    /* eslint-enable id-length */
});

Promise.resolve().then(function () {
    if (cli.input.length) {
        return Object.assign({}, {
            config: cli.flags
        }, {
            files: cli.input
        });
    }

    // For stdin
    return Promise.resolve({});
}).then(function (options) {
    if (!options.files) {
        cli.showHelp();
    }

    if (!options.config.dest) {
        return Promise.reject('Require `--dest` options');
    }

    if (options.config.destCssTemplate) {
        options.config.css = true;

        var extname = _path2.default.extname(options.config.destCssTemplate);

        options.config.cssFormat = extname ? extname.slice(1, extname.length) : 'css';
    }

    return (0, _standalone2.default)(options).then(function (result) {
        return Promise.resolve(result);
    }).then(function (result) {
        result.config = Object.assign({}, options.config);

        return Promise.resolve(result);
    });
}).then(function (result) {
    var fontName = result.config.fontName;

    return new Promise(function (resolve, reject) {
        (0, _mkdirp2.default)(_path2.default.resolve(result.config.dest), function (error) {
            if (error) {
                return reject(error);
            }

            return resolve();
        });
    }).then(function () {
        var cssTemplateDirectory = _path2.default.resolve(_path2.default.dirname(result.config.destCssTemplate));

        return new Promise(function (resolve, reject) {
            (0, _mkdirp2.default)(cssTemplateDirectory, function (error) {
                if (error) {
                    return reject(error);
                }

                return resolve();
            });
        });
    }).then(function () {
        return Promise.all(Object.keys(result).map(function (type) {
            if (type === 'config') {
                return Promise.resolve();
            }

            var content = result[type];
            var destFilename = null;

            if (type !== 'css') {
                destFilename = _path2.default.resolve(_path2.default.join(result.config.dest, fontName + '.' + type));
            } else {
                destFilename = _path2.default.resolve(result.config.destCssTemplate);
            }

            return new Promise(function (resolve, reject) {
                _fs2.default.writeFile(destFilename, content, function (error) {
                    if (error) {
                        return reject(new Error(error));
                    }

                    return resolve();
                });
            });
        }));
    }).then(function () {
        return Promise.resolve(result);
    });
}).catch(function (error) {
    console.log(error.stack); // eslint-disable-line no-console
    process.exit(error.code || 1); // eslint-disable-line no-process-exit
});