'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

exports.default = function () {
    var _ref = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];

    var files = _ref.files;
    var config = _ref.config;

    if (!files) {
        return Promise.reject(new Error('You must pass webfont a `files` glob'));
    }

    var options = Object.assign({}, {
        fontName: 'webfont',
        formats: ['svg', 'ttf', 'eot', 'woff', 'woff2'],
        css: false,
        srcCssTemplate: null,
        cssFormat: 'css',
        cssTemplateClassName: null,
        cssTemplateFontPath: './',
        cssTemplateFontName: null,
        formatOptions: {
            ttf: {
                copyright: null,
                ts: Math.round(Date.now() / 1000)
            }
        },
        metadataProvider: null,
        fontId: null,
        fontStyle: '',
        fontWeight: '',
        fixedWidth: false,
        centerHorizontally: false,
        normalize: false,
        fontHeight: null,
        round: 10e12,
        descent: 0,
        ascent: undefined, // eslint-disable-line no-undefined
        startUnicode: 0xEA01,
        prependUnicode: false,
        metadata: null,
        log: console.log // eslint-disable-line no-console
    }, config);

    if (!options.verbose) {
        options.log = function () {}; // eslint-disable-line no-empty-function
    }

    if (!options.fontId) {
        options.fontId = options.fontName;
    }

    var glyphs = [];

    return (0, _globby2.default)([].concat(files)).then(function (foundFiles) {
        var filteredFiles = foundFiles.filter(function (foundFile) {
            return _path2.default.extname(foundFile) === '.svg';
        });

        if (filteredFiles.length === 0) {
            return Promise.reject(new Error('Files glob patterns specified did not match any files'));
        }

        return Promise.resolve(filteredFiles);
    }).then(function (foundFiles) {
        return svgIcons2svgFontFn(foundFiles, options, glyphs);
    }).then(function (result) {
        return svg2ttfFn(result, options);
    })
    // maybe add ttfautohint
    .then(function (result) {
        if (options.formats.indexOf('eot') === -1) {
            return Promise.resolve(result);
        }

        return ttf2eotFn(result);
    }).then(function (result) {
        if (options.formats.indexOf('woff') === -1) {
            return Promise.resolve(result);
        }

        return ttf2woffFn(result, options);
    }).then(function (result) {
        if (options.formats.indexOf('woff2') === -1) {
            return Promise.resolve(result);
        }

        return ttf2woff2Fn(result, options);
    }).then(function (result) {
        if (!options.css) {
            return Promise.resolve(result);
        }

        if (!options.srcCssTemplate) {
            _nunjucks2.default.configure(_path2.default.join(__dirname, '../'));

            options.srcCssTemplate = _path2.default.join(__dirname, '../templates/template.' + options.cssFormat + '.tpl');
        }

        if (!options.cssTemplateClassName) {
            options.cssTemplateClassName = options.fontName;
        }

        if (!options.cssTemplateFontName) {
            options.cssTemplateFontName = options.fontName;
        }

        var nunjucksOptions = Object.assign({}, {
            glyphs: glyphs
        }, options, {
            className: options.cssTemplateClassName,
            fontPath: options.cssTemplateFontPath,
            fontName: options.cssTemplateFontName
        });

        result.css = _nunjucks2.default.render(_path2.default.resolve(options.srcCssTemplate), nunjucksOptions);

        return result;
    }).then(function (result) {
        if (options.formats.indexOf('svg') === -1) {
            delete result.svg;
        }

        if (options.formats.indexOf('ttf') === -1) {
            delete result.ttf;
        }

        return Promise.resolve(result);
    }).catch(function (error) {
        throw error;
    });
};

var _metadata = require('svgicons2svgfont/src/metadata');

var _metadata2 = _interopRequireDefault(_metadata);

var _filesorter = require('svgicons2svgfont/src/filesorter');

var _filesorter2 = _interopRequireDefault(_filesorter);

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _globby = require('globby');

var _globby2 = _interopRequireDefault(_globby);

var _nunjucks = require('nunjucks');

var _nunjucks2 = _interopRequireDefault(_nunjucks);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _svg2ttf = require('svg2ttf');

var _svg2ttf2 = _interopRequireDefault(_svg2ttf);

var _svgicons2svgfont = require('svgicons2svgfont');

var _svgicons2svgfont2 = _interopRequireDefault(_svgicons2svgfont);

var _ttf2eot = require('ttf2eot');

var _ttf2eot2 = _interopRequireDefault(_ttf2eot);

var _ttf2woff = require('ttf2woff');

var _ttf2woff2 = _interopRequireDefault(_ttf2woff);

var _ttf2woff3 = require('ttf2woff2');

var _ttf2woff4 = _interopRequireDefault(_ttf2woff3);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function svgIcons2svgFontFn(files, options) {
    var glyphs = arguments.length <= 2 || arguments[2] === undefined ? [] : arguments[2];

    var fontStream = (0, _svgicons2svgfont2.default)(options);
    var metadataProvider = options.metadataProvider || (0, _metadata2.default)({
        startUnicode: options.startUnicode,
        prependUnicode: options.prependUnicode
    });

    var sortedFiles = files.sort(function (fileA, fileB) {
        return (0, _filesorter2.default)(fileA, fileB);
    });

    var ret = {
        svg: ''
    };

    return Promise.all(sortedFiles.map(function (srcPath) {
        return new Promise(function (resolve, reject) {
            metadataProvider(srcPath, function (error, metadata) {
                if (error) {
                    return reject(new Error(error));
                }

                var glyph = _fs2.default.createReadStream(srcPath);

                glyph.metadata = metadata;
                fontStream.write(glyph);

                glyphs.push(metadata);

                return resolve(glyph);
            });
        });
    })).then(function () {
        fontStream.end();

        return new Promise(function (resolve, reject) {
            fontStream.on('error', function (error) {
                return reject(new Error(error));
            }).on('data', function (data) {
                ret.svg += data;
            }).on('end', function () {
                return resolve(ret);
            });
        });
    });
}

function svg2ttfFn(result, options) {
    return new Promise(function (resolve) {
        result.ttf = (0, _svg2ttf2.default)(result.svg, options.ttf);

        return resolve(result);
    });
}

function ttf2eotFn(result) {
    return new Promise(function (resolve) {
        result.eot = (0, _ttf2eot2.default)(result.ttf.buffer);

        return resolve(result);
    });
}

function ttf2woffFn(result, options) {
    return new Promise(function (resolve) {
        result.woff = (0, _ttf2woff2.default)(result.ttf.buffer, options);

        return resolve(result);
    });
}

function ttf2woff2Fn(result, options) {
    return new Promise(function (resolve) {
        result.woff2 = (0, _ttf2woff4.default)(result.ttf.buffer, options);

        return resolve(result);
    });
}